#!/bin/bash

###########
# Author: christos faloutsos
# Date: 5/26/2011
###########

# Goal: do ICA on a CSV file
# and plot the pair-plot of the S-matrix


pname=`basename $0`
usage="USAGE: $pname [-h -v -n<n_components> -s<starting_column> ] filename "
longusage=$'    -s <starting_column> (starting from 1)
    -n <n_compoments> number of ICA components
    -w <min_weight>  for heavy-hitters
    -v for verbose
    -h for help'

ncomp=4 # number of ICA components
start_col=2
min_weight=1000 # row sum > min_weight, to be a heavy-hitter

while getopts  "s:n:w:hv" flag
do
    # echo "$flag" $OPTIND $OPTARG

    case "$flag" in
	n) if [ $verbose ]; then
	     echo "***Flag $flag, arg  $OPTARG"
	   fi 
	   ncomp=$OPTARG ;;
	s) if [ $verbose ]; then
	     echo "***Flag $flag, arg  $OPTARG"
	   fi 
	   start_col=$OPTARG ;;
	w) if [ $verbose ]; then
	     echo "***Flag $flag, arg  $OPTARG"
	   fi 
	   min_weight=$OPTARG ;;
	h) if [ $verbose ]; then
	       echo "***Flag $flag"
	   fi 
	   echo $usage 
	   echo "$longusage"
	   exit;;
	v) echo "***Flag $flag"
	   verbose=true ;;
	?) echo "Flag $flag unknown - ignored"
	   echo $usage ;;
    esac
done

if [ $verbose ]; then
    echo "hi world! with n=$ncomp s=$start_col w=${min_weight}"
fi 

shift $(($OPTIND - 1))



if [ $# -ne 1 ] ; then
    echo "ERROR: $pname expects one file: " 
    echo $usage
    exit 1
fi

fname=$1

echo "$0 working on $fname"


##################################


# ncomp=4 # number of ICA components
# start_col=2
# min_weight=1000 # row sum > min_weight, to be a heavy-hitter

# end_col=5
outfile=${fname%.csv}.pdf

R --no-save <<EOR
#install.packages('fastICA')
library(fastICA)
# a <- read.csv("${fname}", header=FALSE, sep=",", quote="\\"", strip.white=TRUE, row.names=1)
a <- read.csv("${fname}", header=FALSE)

nCols=dim(a)[2]
## nCols
x <- a[  , ${start_col}:nCols]
## x

pdf("${outfile}")

f <- fastICA( as.matrix(x), n.comp= ${ncomp} )

# spot the heavyhitters and plot ONLY them
hh <-rowSums(x) > ${min_weight}
## hh
toPlot <- f\$S[hh,]
## toPlot

pairs( toPlot , main="${fname}", pch=20 ) # plots dots
dev.off()
EOR

# okular $outfile
# acroread $outfile
open $outfile

